/**
 * Admin tool: AddonFileEditor
 *
 * This tool allows you to "edit", "delete", "create", "upload" or "backup" files of installed
 * Add-ons such as modules, templates and languages via the WebsiteBaker backend. This enables
 * you to perform small modifications on installed Add-ons without downloading the files first.
 *
 * This file contains the jQuery code to toggle the section overview.
 *
 * LICENSE: GNU General Public License 3.0
 *
 * @platform    CMS WebsiteBaker 2.8.x
 * @package     AddonFileEditor
 * @author      cwsoft (http://cwsoft.de)
 * @copyright   cwsoft
 * @license     http://www.gnu.org/licenses/gpl-3.0.html
*/

// functions for the jQuery framework
$(document).ready(function() {

    function writeSessionCookie (cookieName, cookieValue) {
        if ('localStorage' in window &&
            (window['localStorage'] !== null)) {
            key = localStorage.getItem(cookieName);
            if (key){
                localStorage.removeItem(cookieName);
            } else {
                localStorage.setItem(cookieName, true);
            }
        }
        else {
            if (cookieValue === "1"){
                document.cookie = escape(cookieName) + "=" + escape(cookieValue) + ";samesite=lax;";
            }
            else{
                document.cookie = escape(cookieName)+"=; max-age=-1;samesite=lax;";
            }
        }
    }

    // only execute on overview page
    if ($('#trigger_modules,#trigger_templates,#trigger_languages').html() === null) return;

    // remove navigation boxes except the very first one (#modules)
    $('.box:not("#modules")').hide();

    // remove links from first navigation box except the first link (Reload)
    $('#modules a:not(:first)').hide();

    // set default or user defined view mode settings
    setViewMode('modules');
    setViewMode('templates');
    setViewMode('languages');

    $('#trigger_modules, #trigger_templates, #trigger_languages').click(function() {
        var $this = $(this);
        if($this.hasClass('clicked')) {
            var id = $this.attr('id').replace(/trigger_/, '');
            writeSessionCookie('AFE_' + id,"0");
//            document.cookie = 'AFE_' + id + '=0';
            setViewMode(id);
            $this.removeClass('clicked');
        } else {
            var id = $this.attr('id').replace(/trigger_/, '');
            writeSessionCookie('AFE_' + id,"1");
//            document.cookie = 'AFE_' + id + '=1';
            setViewMode(id);
            $this.addClass('clicked');
        }
    });

});

function setViewMode(id) {
    // get class (expanded/collapsed)
    var view    = $('#toggle_' + id)
    var trigger = $('#trigger_' + id);
    if ('localStorage' in window &&
        (window['localStorage'] !== null)) {
        key = localStorage.getItem("AFE_"+id);
    }else {
        key = (document.cookie.indexOf('AFE_' + id) === -1);
    }
    // initialize trigger
    if (trigger.attr('class') === '' && key ) {
        // initialize triggers (set collapsed)
        trigger.addClass('expanded');
        view.addClass('hidden');
//        document.cookie = 'AFE_' + id + '=0';
        writeSessionCookie('AFE_' + id,"0");
    } else if (key || (document.cookie.indexOf('AFE_' + id + '=1') > -1)) {
        // toggle from collapsed to expanded
        trigger.removeClass('expanded').addClass('collapsed');
        view.removeClass('hidden');
    } else {
        // toggle from expanded to collapsed
        trigger.removeClass('collapsed').addClass('expanded');
        view.addClass('hidden');
    }
}